diff -urN linux-2.4.21/arch/arm/mach-clps711x/zipline.c linux-2.4.21.zipit/arch/arm/mach-clps711x/zipline.c
--- linux-2.4.21/arch/arm/mach-clps711x/zipline.c	1969-12-31 19:00:00.000000000 -0500
+++ linux-2.4.21.zipit/arch/arm/mach-clps711x/zipline.c	2004-04-21 12:01:30.000000000 -0400
@@ -0,0 +1,117 @@
+/*
+ *  linux/arch/arm/mach-clps711x/zipline.c
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ */
+#include <linux/sched.h>
+#include <linux/mm.h>
+#include <linux/init.h>
+#include <linux/types.h>
+#include <linux/string.h>
+
+#include <asm/setup.h>
+#include <asm/hardware.h>
+#include <asm/pgtable.h>
+#include <asm/page.h>
+#include <asm/mach-types.h>
+#include <asm/mach/arch.h>
+#include <asm/mach/map.h>
+
+extern void zipline_init_irq(void);
+extern void clps711x_init_irq(void);
+extern void clps711x_map_io(void);
+
+#define MB1 1048576	/* one megabyte == size of an MMU section */
+
+/*
+ * The on-chip registers are given a size of 1MB so that a section can
+ * be used to map them; this saves a page table.  This is the place to
+ * add mappings for ROM, expansion memory, PCMCIA, etc.  (if static
+ * mappings are chosen for those areas).
+ *
+ * Here is a physical memory map (to be fleshed out later):
+ *
+ * Physical Address  Size  Description
+ * ----------------- ----- ---------------------------------
+ * c0000000-c001ffff 128KB reserved for video RAM [1]
+ * c0020000-c0023fff  16KB parameters (see Documentation/arm/Setup)
+ * c0024000-c0027fff  16KB swapper_pg_dir (task 0 page directory)
+ * c0028000-...            kernel image (TEXTADDR)
+ *
+ * [1] Unused pages should be given back to the VM; they are not yet.
+ *     The parameter block should also be released (not sure if this
+ *     happens).
+ */
+static struct map_desc zipline_io_desc[] __initdata = {
+ /* virtual, physical, length, domain, r, w, c, b */
+
+ /* Frame Buffer memory */
+ { ZIPLINE_VIRT_SDRAM_FRAMEBUFFER, ZIPLINE_PHYS_SDRAM_FRAMEBUFFER, MB1, DOMAIN_IO, 1, 1, 1, 1 },
+
+ /* Internal SRAM and CS8900A Ethernet chip */
+ { ZIPLINE_VIRT_SRAM_CONFIG, ZIPLINE_PHYS_SRAM_CONFIG, MB1, DOMAIN_IO, 1, 1, 1, 1 },
+ { ZIPLINE_VIRT_CS8900A, ZIPLINE_PHYS_CS8900A, MB1, DOMAIN_IO, 1, 1, 0, 0 },
+ /* Need to add NAND flash, PS/2, IDE, Parallel and others */
+
+ /* flash banks */
+ { ZIPLINE_VIRT_FLASH, ZIPLINE_PHYS_FLASH, MB1 * 16, DOMAIN_IO, 1, 1, 0, 0 },
+
+ { ZIPLINE_VIRT_RADIO_IO, ZIPLINE_PHYS_RADIO_IO, MB1, DOMAIN_IO, 1, 1, 0, 0 },
+ { ZIPLINE_VIRT_RADIO_CONFIG, ZIPLINE_PHYS_RADIO_CONFIG, MB1, DOMAIN_IO, 1, 1, 0, 0 },
+
+ LAST_DESC
+};
+
+void __init zipline_map_io(void)
+{
+        clps711x_map_io();
+        iotable_init(zipline_io_desc);
+}
+
+static void __init
+fixup_zipline(struct machine_desc *desc, struct param_struct *params,
+	    char **cmdline, struct meminfo *mi)
+{
+/*
+	mi->nr_banks=1;
+	mi->end = 0xc0FFFFFF;
+	mi->bank[0].start = 0xc0000000;
+	mi->bank[0].size = 0x01000000;
+	mi->bank[0].node = 0;
+
+#ifdef CONFIG_BLK_DEV_INITRD
+	params->u1.s.page_size=0x1000;
+	params->u1.s.nr_pages=0x1000;
+	params->u1.s.ramdisk_size=0xC00;
+	params->u1.s.flags=FLAG_RDPROMPT;
+	params->u1.s.rootdev=0x100;
+	params->u1.s.initrd_start=__phys_to_virt(0xc0600000);
+	params->u1.s.initrd_size=0x200000;
+	params->u1.s.rd_start=0x0;
+	strcpy(params->commandline, "");
+#endif
+*/
+}
+
+
+MACHINE_START(ZIPLINE, "ZipLine - Cirrus Logic 7312")
+	MAINTAINER("Aeronix, Inc.")
+        BOOT_MEM(0xc0000000, 0x80000000, 0xff000000)
+	BOOT_PARAMS(0xc0020000)
+	FIXUP(fixup_zipline)
+	MAPIO(zipline_map_io)
+/* zipline TODO INITIRQ(ep7312_init_irq)*/
+	INITIRQ(clps711x_init_irq)
+MACHINE_END

