diff -urN linux-2.4.21/drivers/net/cs89x0.c linux-2.4.21.zipit/drivers/net/cs89x0.c
--- linux-2.4.21/drivers/net/cs89x0.c	2005-02-07 11:17:13.134293474 -0500
+++ linux-2.4.21.zipit/drivers/net/cs89x0.c	2004-03-15 14:52:28.000000000 -0500
@@ -164,6 +164,10 @@
 static unsigned int netcard_portlist[] __initdata =
    { 0x0300, 0};
 static unsigned int cs8900_irq_map[] = {1,0,0,0};
+#elif defined(CONFIG_ARCH_ZIPLINE)
+static unsigned int netcard_portlist[] __initdata =
+    { 0xfc000003, 0 };
+static unsigned int cs8900_irq_map[] = {7,0,0,0};
 #else
 static unsigned int netcard_portlist[] __initdata =
    { 0x300, 0x320, 0x340, 0x360, 0x200, 0x220, 0x240, 0x260, 0x280, 0x2a0, 0x2c0, 0x2e0, 0};
@@ -269,7 +273,7 @@
 __setup("cs89x0_media=", media_fn);
 #endif
 
-
+
 /* Check for a network adaptor of this type, and return '0' iff one exists.
    If dev->base_addr == 0, probe all likely locations.
    If dev->base_addr == 1, always return failure.
@@ -277,7 +281,6 @@
    (detachable devices only).
    Return 0 on success.
    */
-
 int __init cs89x0_probe(struct net_device *dev)
 {
 	int i;
@@ -411,6 +414,7 @@
         }
 	lp = (struct net_local *)dev->priv;
 
+#ifndef CONFIG_ARCH_ZIPLINE
 	/* Grab the region so we can find another board if autoIRQ fails. */
 	if (!request_region(ioaddr & ~3, NETCARD_IO_EXTENT, dev->name)) {
 		printk(KERN_ERR "%s: request_region(0x%x, 0x%x) failed\n",
@@ -418,6 +422,7 @@
 		retval = -EBUSY;
 		goto out1;
 	}
+#endif /* CONFIG_ARCH_ZIPLINE */
 
 #ifdef CONFIG_SH_HICOSH4
 	/* truely reset the chip */
@@ -502,7 +507,7 @@
 		cnt = (*confd++ & 0x00ff) >> 1;
 		while (--cnt > 0) {
 			__u16 j = *confd++;
-			
+
 			switch (j & 0x0fff) {
 			case PP_IA:
 				for (i = 0; i < ETH_ALEN/2; i++) {
@@ -518,7 +523,7 @@
 	} else
 #endif
 
-        if ((readreg(dev, PP_SelfST) & (EEPROM_OK | EEPROM_PRESENT)) == 
+        if ((readreg(dev, PP_SelfST) & (EEPROM_OK | EEPROM_PRESENT)) ==
 	      (EEPROM_OK|EEPROM_PRESENT)) {
 	        /* Load the MAC. */
 		for (i=0; i < ETH_ALEN/2; i++) {
@@ -527,17 +532,17 @@
 		        dev->dev_addr[i*2] = Addr & 0xFF;
 		        dev->dev_addr[i*2+1] = Addr >> 8;
 		}
-   
-	   	/* Load the Adapter Configuration. 
-		   Note:  Barring any more specific information from some 
-		   other source (ie EEPROM+Schematics), we would not know 
-		   how to operate a 10Base2 interface on the AUI port. 
-		   However, since we  do read the status of HCB1 and use 
-		   settings that always result in calls to control_dc_dc(dev,0) 
-		   a BNC interface should work if the enable pin 
-		   (dc/dc converter) is on HCB1. It will be called AUI 
+
+	   	/* Load the Adapter Configuration.
+		   Note:  Barring any more specific information from some
+		   other source (ie EEPROM+Schematics), we would not know
+		   how to operate a 10Base2 interface on the AUI port.
+		   However, since we  do read the status of HCB1 and use
+		   settings that always result in calls to control_dc_dc(dev,0)
+		   a BNC interface should work if the enable pin
+		   (dc/dc converter) is on HCB1. It will be called AUI
 		   however. */
-	   
+
 		lp->adapter_cnf = 0;
 		i = readreg(dev, PP_LineCTL);
 		/* Preserve the setting of the HCB1 pin. */
@@ -554,22 +559,22 @@
 			lp->adapter_cnf |=  A_CNF_AUI | A_CNF_MEDIA_AUI;
 		/* Check if the card is in Auto mode. */
 		if ((i & (AUI_ONLY | AUTO_AUI_10BASET)) == AUTO_AUI_10BASET)
-			lp->adapter_cnf |=  A_CNF_AUI | A_CNF_10B_T | 
+			lp->adapter_cnf |=  A_CNF_AUI | A_CNF_10B_T |
 			A_CNF_MEDIA_AUI | A_CNF_MEDIA_10B_T | A_CNF_MEDIA_AUTO;
-		
+
 		if (net_debug > 1)
 			printk(KERN_INFO "%s: PP_LineCTL=0x%x, adapter_cnf=0x%x\n",
 					dev->name, i, lp->adapter_cnf);
 
 		/* IRQ. Other chips already probe, see below. */
-		if (lp->chip_type == CS8900) 
+		if (lp->chip_type == CS8900)
 			lp->isa_config = readreg(dev, PP_CS8900_ISAINT) & INT_NO_MASK;
-	   
+
 		printk( "[Cirrus EEPROM] ");
 	}
 
         printk("\n");
-   
+
 	/* First check to see if an EEPROM is attached. */
 #ifdef CONFIG_SH_HICOSH4 /* no EEPROM on HiCO, don't hazzle with it here */
 	if (1) {
@@ -577,20 +582,39 @@
 	} else
 #endif
 	if ((readreg(dev, PP_SelfST) & EEPROM_PRESENT) == 0)
+#ifdef CONFIG_ARCH_ZIPLINE
+	{
+		char ZIPLINE[] = { 0,1,2,3,4,5 };
+
+		for (i=0; i < ETH_ALEN; i++)
+		{
+		        dev->dev_addr[i] = ZIPLINE[i];
+		}
+		printk(KERN_WARNING "cs89x0: No EEPROM, hacked in ETHERNET ADDRESS....\n");
+		
+		if (!lp->auto_neg_cnf)
+			lp->auto_neg_cnf = 1;
+
+		/* Store adapter configuration */
+		lp->adapter_cnf =  A_CNF_MEDIA_10B_T|A_CNF_10B_T|A_CNF_DC_DC_POLARITY;
+		lp->isa_config = 0;
+	}
+#else
 		printk(KERN_WARNING "cs89x0: No EEPROM, relying on command line....\n");
+#endif /* CONFIG_ARCH_ZIPLINE */
 	else if (get_eeprom_data(dev, START_EEPROM_DATA,CHKSUM_LEN,eeprom_buff) < 0) {
 		printk(KERN_WARNING "\ncs89x0: EEPROM read failed, relying on command line.\n");
         } else if (get_eeprom_cksum(START_EEPROM_DATA,CHKSUM_LEN,eeprom_buff) < 0) {
 		/* Check if the chip was able to read its own configuration starting
 		   at 0 in the EEPROM*/
 		if ((readreg(dev, PP_SelfST) & (EEPROM_OK | EEPROM_PRESENT)) !=
-		    (EEPROM_OK|EEPROM_PRESENT)) 
+		    (EEPROM_OK|EEPROM_PRESENT))
                 	printk(KERN_WARNING "cs89x0: Extended EEPROM checksum bad and no Cirrus EEPROM, relying on command line\n");
-		   
+
         } else {
 		/* This reads an extended EEPROM that is not documented
 		   in the CS8900 datasheet. */
-		
+
                 /* get transmission control word  but keep the autonegotiation bits */
                 if (!lp->auto_neg_cnf) lp->auto_neg_cnf = eeprom_buff[AUTO_NEG_CNF_OFFSET/2];
                 /* Store adapter configuration */
@@ -655,7 +679,7 @@
 				printk("\ncs89x0: invalid ISA interrupt number %d\n", i);
 			else
 				i = cs8900_irq_map[i];
-			
+
 			lp->irq_map = CS8900_IRQ_MAP; /* fixed IRQ map for CS8900 */
 		} else {
 			int irq_map_buff[IRQ_MAP_LEN/2];
@@ -671,7 +695,7 @@
 			dev->irq = i;
 	}
 
-	printk(" IRQ %d", dev->irq);
+	printk(" IRQ %d, 0x%04X", dev->irq, lp->irq_map);
 
 #if ALLOW_DMA
 	if (lp->use_dma) {
@@ -708,7 +732,9 @@
 		printk("cs89x0_probe1() successful\n");
 	return 0;
 out2:
+#ifndef CONFIG_ARCH_ZIPLINE
 	release_region(ioaddr & ~3, NETCARD_IO_EXTENT);
+#endif /* CONFIG_ARCH_ZIPLINE */
 out1:
 	kfree(dev->priv);
 	dev->priv = 0;
@@ -716,7 +742,7 @@
 	return retval;
 }
 
-
+
 /*********************************
  * This page contains DMA routines
 **********************************/
@@ -1815,7 +1841,9 @@
                 kfree(dev_cs89x0.priv);
                 dev_cs89x0.priv = NULL;	/* gets re-allocated by cs89x0_probe1 */
                 /* If we don't do this, we can't re-insmod it later. */
+#ifndef CONFIG_ARCH_ZIPLINE
                 release_region(dev_cs89x0.base_addr, NETCARD_IO_EXTENT);
+#endif /* CONFIG_ARCH_ZIPLINE */
         }
 }
 #endif /* MODULE */

