diff -urN linux-2.4.21/include/asm-arm/hardware/ep7312.h linux-2.4.21.zipit/include/asm-arm/hardware/ep7312.h
--- linux-2.4.21/include/asm-arm/hardware/ep7312.h	2005-02-07 11:17:14.000000000 -0500
+++ linux-2.4.21.zipit/include/asm-arm/hardware/ep7312.h	2004-04-20 12:46:41.000000000 -0400
@@ -4,7 +4,7 @@
  *  This file contains the hardware definitions of the EP7312 internal
  *  registers.
  *
- *  Copyright (C) 2003 Roland A.I. Rosier <roland.rosier@lycos.co.uk>.
+ *  Copyright (C) 2000 Deep Blue Solutions Ltd.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,49 +18,114 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 #ifndef __ASM_HARDWARE_EP7312_H
 #define __ASM_HARDWARE_EP7312_H
 
 #include <linux/config.h>
 
-/*
- * Include CLPS7111 and EP7212 register definitions
- */
-
 #include <asm/hardware/clps7111.h>
-#include <asm/hardware/ep7212.h>
 
 /*
  * define EP7312_BASE to be the base address of the region
  * you want to access.
  */
 
-#define EP7312_PHYS_BASE        (0x80000000)
+#define EP7312_PHYS_BASE	(0x80000000)
+#define EP7312_BASE		(0xFF000000)
+
+#ifndef __ASSEMBLY__
+#define ep_readl(off)		__raw_readl(EP7312_BASE + (off))
+#define ep_writel(val,off)	__raw_writel(val, EP7312_BASE + (off))
+
+#define ep_readb(off)		__raw_readb(EP7312_BASE + (off))
+#define ep_writeb(val,off)	__raw_writeb(val, EP7312_BASE + (off))
+#endif
+
+/*
+ * These registers are specific to the EP7312 only
+ */
+#define DAIR			0x2000
+#define DAIR0			0x2040
+#define DAIDR1			0x2080
+#define DAIDR2			0x20c0
+#define DAISR			0x2100
+#define SYSCON3			0x2200
+#define INTSR3			0x2240
+#define INTMR3			0x2280
+#define LEDFLSH			0x22c0
+#define SDCONF			0x2300
+#define SDRFPR			0x2340
+#define DAI64FS			0x2600
+
+#define DAIR_DAIEN		(1 << 16)
+#define DAIR_ECS		(1 << 17)
+#define DAIR_LCTM		(1 << 19)
+#define DAIR_LCRM		(1 << 20)
+#define DAIR_RCTM		(1 << 21)
+#define DAIR_RCRM		(1 << 22)
+#define DAIR_LBM		(1 << 23)
+
+#define DAIDR2_FIFOEN		(1 << 15)
+#define DAIDR2_FIFOLEFT		(0x0d << 16)
+#define DAIDR2_FIFORIGHT	(0x11 << 16)
+
+#define DAISR_RCTS		(1 << 0)
+#define DAISR_RCRS		(1 << 1)
+#define DAISR_LCTS		(1 << 2)
+#define DAISR_LCRS		(1 << 3)
+#define DAISR_RCTU		(1 << 4)
+#define DAISR_RCRO		(1 << 5)
+#define DAISR_LCTU		(1 << 6)
+#define DAISR_LCRO		(1 << 7)
+#define DAISR_RCNF		(1 << 8)
+#define DAISR_RCNE		(1 << 9)
+#define DAISR_LCNF		(1 << 10)
+#define DAISR_LCNE		(1 << 11)
+#define DAISR_FIFO		(1 << 12)
+
+#define SYSCON3_ADCCON		(1 << 0)
+#define SYSCON3_DAISEL		(1 << 3)
+#define SYSCON3_ADCCKNSEN	(1 << 4)
+#define SYSCON3_FASTWAKE	(1 << 8)
+#define SYSCON3_DAIEN		(1 << 9)
+
+#define SDCONF_ACTIVE		(1 << 10)
+#define SDCONF_CLKCTL		(1 << 9)
+#define SDCONF_WIDTH_4		(0 << 7)
+#define SDCONF_WIDTH_8		(1 << 7)
+#define SDCONF_WIDTH_16		(2 << 7)
+#define SDCONF_WIDTH_32		(3 << 7)
+#define SDCONF_SIZE_16		(0 << 5)
+#define SDCONF_SIZE_64		(1 << 5)
+#define SDCONF_SIZE_128		(2 << 5)
+#define SDCONF_SIZE_256		(3 << 5)
+#define SDCONF_CASLAT_2		(2)
+#define SDCONF_CASLAT_3		(3)
+
+#define DAI64FS_I2SF64		(1 << 0)
+#define DAI64FS_AUDIOCLKEN	(1 << 1)
+//#define DAI64FS_AUDCLK11MHZ	(1 << 2)
+#define DAI64FS_AUDEXTCLK	(1 << 2)
+#define DAI64FS_MCLKEN		(1 << 3)
+//#define DAI64FS_22KHZ		(4 << 8)
+//#define DAI64FS_44KHZ		(2 << 8)
+
+#define DAI64FS_KHZMASK		(0x7F << 8)
+
+#ifdef CONFIG_ZIPLINE_90MHZ
+
+#define DAI64FS_32KHZ		(11 << 8)
+#define DAI64FS_44KHZ		(8 << 8)
+#define DAI64FS_48KHZ		((2 << 8) | DAI64FS_AUDEXTCLK)
+
+#else
+
+#define DAI64FS_32KHZ		(9 << 8)
+#define DAI64FS_44KHZ		((2 << 8) | DAI64FS_AUDEXTCLK)
+#define DAI64FS_48KHZ		(6 << 8)
 
-#define SDCONF   0x2300	/* SDRAM Configuration register */
-#define SDRFPR   0x2340	/* SDRAM Refresh period register */
-#define UNIQID   0x2440	/* 32 bit unique ID for the EP73xx device */
-#define DAI64FS  0x2600	/* DAI 64Fs Control Register */
-#define PLLW     0x2610	/* Write Register for PLL Multiplier */
-#define PLLR     0xA5A8	/* Read Register for PLL Multiplier */
-#define RANDID0	 0x2700	/* Bits 31-0 of 128-bit random ID */
-#define RANDID1	 0x2704	/* Bits 63-32 of 128-bit random ID */
-#define RANDID2	 0x2708	/* Bits 95-64 of 128-bit random ID */
-#define RANDID3	 0x270C	/* Bits 127-96 of 128-bit random ID */
-
-#define SDCONF_ACTIVE           (1 << 10)
-#define SDCONF_CLKCTL           (1 << 9)
-#define SDCONF_WIDTH_4          (0 << 7)
-#define SDCONF_WIDTH_8          (1 << 7)
-#define SDCONF_WIDTH_16         (2 << 7)
-#define SDCONF_WIDTH_32         (3 << 7)
-#define SDCONF_SIZE_16          (0 << 5)
-#define SDCONF_SIZE_64          (1 << 5)
-#define SDCONF_SIZE_128         (2 << 5)
-#define SDCONF_SIZE_256         (3 << 5)
-#define SDCONF_CASLAT_2         (2)
-#define SDCONF_CASLAT_3         (3)
+#endif
 
 #endif /* __ASM_HARDWARE_EP7312_H */

