diff -urN -X dontdiff linux-2.4.21/drivers/char/zipline_keyb.c linux-2.4.21.32/drivers/char/zipline_keyb.c
--- linux-2.4.21/drivers/char/zipline_keyb.c	1969-12-31 17:00:00.000000000 -0700
+++ linux-2.4.21.32/drivers/char/zipline_keyb.c	2005-09-17 20:55:06.000000000 -0600
@@ -0,0 +1,377 @@
+/*
+ * drivers/char/zipline_keyb.c
+ *
+ * Copyright (C) 2005 Ken McGuire
+ * 
+ * Keyboard driver for Zipit IM device, based on drivers/char/clps711x_keyb.c
+ *   Lots trimmed since this is a reduction in scope from the original.
+ *
+ * * Copyright (C) 2001 Thomas Gleixner <gleixner@autronix.de>
+ *
+ * based on drivers/edb7211_keyb.c, which is copyright (C) 2000 Bluemug Inc.
+ * 
+ * Keyboard driver for ARM Linux on EP7xxx and CS89712 processors
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 2 as
+ * published by the Free Software Foundation. See the file COPYING 
+ * in the main directory of this archive for more details.
+ *
+ *
+ * Hardware: 
+ *
+ * matrix scan keyboards based on EP7209,7211,7212,7312,Zipit (zipline) and CS89712 
+ * on chip keyboard scanner.		
+ * Adaption for different machines is done in init function.
+ *
+ * Basic Function:
+ *  Changed because Zipit was stupid in their keyboard design. The Standby switch is
+ *  attached to PA:7 pulled to vcc by a resistor and connedted to gnd when the lid
+ *  is closed. This causes an interrupt to always occur if the keyboard is set up as
+ *  below. I left this description in.
+ * 
+ * Basicly the driver is interrupt driven. It sets all column drivers	
+ * high. If any key is pressed, a interrupt occures. Now a seperate scan of
+ * each column is done. This scan is timer based, because we use a keyboard 
+ * interface with decoupling capacitors (neccecary if you want to survive 
+ * EMC compliance tests). Always one line is set high. When next timer event 
+ * occures the scan data on port A are valid. This makes also sure, that no
+ * spurious keys are scanned. The kbd int on these CPU's is not deglitched!
+ * After scanning all columns, we switch back to int mode, if no key is
+ * pressed. If any is pressed we reschedule the scan within a programmable
+ * delay. If we would switch back to interrupt mode as long as a key is pressed,
+ * we come right back to the interrupt, because the int. is level triggered !
+ * The timer based scan of the seperate columns can also be done in one
+ * timer event (set fastscan to 1).
+ *
+ * Summary:
+ * The design of this keyboard controller chip is stupid at all !
+ *  (I don't think so... (km))
+ * 
+ * Matrix translation:
+ * The matrix translation table is based on standard XT scancodes.
+ *
+ * HandyKey:
+ *  Removed...
+ * 
+ * Read access gives back the actual state of the keyboard scan function	
+ *	k:1	Disabled
+ *	k:0	Enabled-- No change
+ * Write access has two functions. Changing the user function mode and applying
+ * a different scancode translation table.
+ * Syntax is: 	k:1	disable keyboard scan timer function
+ *		k:0	no change
+ */
+
+#include <linux/config.h>
+#include <linux/sched.h>
+#include <linux/interrupt.h>
+#include <linux/tty.h>
+#include <linux/tty_flip.h>
+#include <linux/mm.h>
+#include <linux/slab.h>
+#include <linux/ptrace.h>
+#include <linux/signal.h>
+#include <linux/timer.h>
+#include <linux/tqueue.h>
+#include <linux/random.h>
+#include <linux/ctype.h>
+#include <linux/init.h>
+#include <linux/kbd_ll.h>
+#include <linux/kbd_kern.h>
+#include <linux/delay.h>
+#include <linux/proc_fs.h>
+
+#include <asm/bitops.h>
+#include <asm/keyboard.h>
+#include <asm/irq.h>
+#include <asm/hardware.h>
+#include <asm/uaccess.h>
+
+#include <asm/io.h>
+#include <asm/system.h>
+
+   void zipline_kbd_init_hw(void);
+
+/*
+ * Values for the keyboard column scan control register.
+ */
+#define KBSC_HI	    0x0	    /*   All driven high */
+#define KBSC_LO	    0x1	    /*   All driven low */
+#define KBSC_X	    0x2	    /*   All high impedance */
+#define KBSC_COL0   0x8	    /*   Column 0 high, others high impedance */
+#define KBSC_COL1   0x9	    /*   Column 1 high, others high impedance */
+#define KBSC_COL2   0xa	    /*   Column 2 high, others high impedance */
+#define KBSC_COL3   0xb	    /*   Column 3 high, others high impedance */
+#define KBSC_COL4   0xc	    /*   Column 4 high, others high impedance */
+#define KBSC_COL5   0xd	    /*   Column 5 high, others high impedance */
+#define KBSC_COL6   0xe	    /*   Column 6 high, others high impedance */
+#define KBSC_COL7   0xf	    /*   Column 7 high, others high impedance */
+
+
+/* Simple translation table for the SysRq keys */
+
+#ifdef CONFIG_MAGIC_SYSRQ
+unsigned char zipline_kbd_sysrq_xlate[128] =
+	"\000\0331234567890-=\177\t"			/* 0x00 - 0x0f */
+	"qwertyuiop[]\r\000as"				/* 0x10 - 0x1f */
+	"dfghjkl;'`\000\\zxcv"				/* 0x20 - 0x2f */
+	"bnm,./\000*\000 \000\201\202\203\204\205"	/* 0x30 - 0x3f */
+	"\206\207\210\211\212\000\000789-456+1"		/* 0x40 - 0x4f */
+	"230\177\000\000\213\214\000\000\000\000\000\000\000\000\000\000" /* 0x50 - 0x5f */
+	"\r\000/";					/* 0x60 - 0x6f */
+#endif
+
+/* 
+ * This table maps row/column keyboard matrix positions to XT scancodes.
+ * It's a default table, which can be overriden by writing to proc/keyboard 
+ */
+
+static unsigned char zip_scancode[64] = 
+{
+/*  Column: 
+  Row       0     1     2     3     4     5     6     7   */
+/*        RtAr  UpAr  LtAr  P2    P1    Stop  Play  Prev  */
+/* A0 */  0x4d, 0x48, 0x4b, 0x3c, 0x3b, 0x02, 0x03, 0x04,
+/*        P     DnAr  O(Oh) P3    Zip!  /-/   Mute  Next  */
+/* A1 */  0x19, 0x50, 0x18, 0x3d, 0x05, 0x00, 0x06, 0x0f,
+/*        I     U     Y     T     R     E     W     Q     */
+/* A2 */  0x17, 0x16, 0x15, 0x14, 0x13, 0x12, 0x11, 0x10,
+/*        K     J     H     G     F     D     S     A     */
+/* A3 */  0x25, 0x24, 0x23, 0x22, 0x21, 0x20, 0x1f, 0x1e,
+/*        M     N     B     V     C     X     Z     ...   */
+/* A4 */  0x32, 0x31, 0x30, 0x2f, 0x2e, 0x2d, 0x2c, 0x01,
+/*        /-/   Menu  .     ,     Spc   Shft  Ctrl  Alt   */ 
+/* A5 */  0x00, 0x07, 0x34, 0x33, 0x39, 0x2a, 0x1d, 0x38,
+/*        /-/   ;     Entr  BS    L     /-/   /-/   /-/   */
+/* A6 */  0x00, 0x27, 0x1c, 0x0e, 0x26, 0x00, 0x00, 0x00,
+/*        /-/   /-/   /-/   /-/   /-/   /-/   /-/   /-/   */
+/* A7 */  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+};
+
+static int keys[8];
+static int new_keys[8];
+static int previous_keys[8];
+
+static int fastscan;
+static int scan_interval;
+static int scan_delay;
+static int last_column; 
+static int key_is_pressed;
+
+static unsigned char *act_scancode;
+
+static 	int	keyb_ena;
+
+static struct tq_struct kbd_process_task;
+static struct timer_list zipline_kbd_timer;
+static struct proc_dir_entry *zipline_keyboard_proc_entry = NULL;
+
+/*
+ * Translate a raw keycode to an XT keyboard scancode.
+ */
+static int zipline_translate(unsigned char scancode, unsigned char *keycode,
+		 		 char raw_mode)
+{
+	*keycode = act_scancode[scancode];
+
+	return 1;
+}
+
+
+/*
+* Check for key and process it	
+*/
+void inline zipline_check(int col, int row) {
+
+	int scode, down;
+	unsigned char kcode;
+	
+	scode = (row<<3) + col;
+	down  = keys[col]>>row & 0x01;
+	kcode = act_scancode[scode];
+
+	if(!kcode)
+		return;
+
+			handle_scancode(scode, down);
+
+		return;	
+}
+
+
+/*
+ * Process the new key data 
+ */
+static void zipline_kbd_process(void* data)
+{
+	int col,row,res;
+
+	for (col = 0; col < 8; col++) {
+		if (( res = previous_keys[col] ^ keys[col]) == 0) 		
+			continue;
+		for(row = 0; row < 8; row++) {
+			if ( ((res >> row) & 0x01) != 0) 
+				zipline_check(col,row);	
+		}				
+	}
+	/* Update the state variables. */
+	memcpy(previous_keys, keys, 8 * sizeof(int));
+
+
+}
+
+static char zipline_unexpected_up(unsigned char scancode)
+{
+	return 0200;
+}
+
+/*
+* Handle timer event, while in pollmode
+*/
+static void zipline_kbd_timeout(unsigned long data)
+{
+	int i;
+	/* 
+	* read bits of actual column or all columns in fastscan-mode
+	*/
+	for (i = 0; i < 8; i++) {
+		new_keys[last_column - KBSC_COL0] = clps_readb(PADR) & 0x7f;
+		key_is_pressed |= new_keys[last_column - KBSC_COL0];
+		last_column = last_column < KBSC_COL7 ? last_column + 1 : KBSC_COL0;
+		clps_writel( (clps_readl(SYSCON1) & ~SYSCON1_KBDSCANMASK) 
+				| last_column, SYSCON1);
+		/*
+		* For fastscan, apply a short delay to settle scanlines
+		* else break and wait for next timeout
+		*/
+		if (fastscan)
+			udelay(5);	
+		else
+			break;
+	}
+
+	/*
+	* reschedule scan
+	*/
+	if (last_column == KBSC_COL0) {
+			zipline_kbd_timer.expires = jiffies + scan_interval;
+			add_timer(&zipline_kbd_timer);
+			key_is_pressed = 0;
+			memcpy(keys, new_keys, 8 * sizeof(int));
+		for (i = 0; i < 8; i++) {
+			if (previous_keys[i] != keys[i]) {
+				queue_task(&kbd_process_task, &tq_timer);
+				return;
+			}
+		}
+	} else {
+		zipline_kbd_timer.expires = jiffies + scan_delay;
+		add_timer(&zipline_kbd_timer);
+	}
+}
+
+static int zipline_kbd_proc_keyboard_read(char *page, char **start, off_t off,
+		int count, int *eof, void *data)
+{
+	if (count < 2) 
+		return -EINVAL;
+
+	return sprintf(page,"k:%d\n",keyb_ena);
+}
+
+static int zipline_kbd_proc_keyboard_write(struct file *file, const char *buffer, 
+		unsigned long count, void *data)
+{
+	unsigned char buf[68];
+
+	if (count < 3|| count > 4)
+		return -EINVAL;
+	if (copy_from_user(buf, buffer, count)) 
+		return -EFAULT;
+	if (buf[1] != ':')
+		return -EINVAL;
+			
+	if (buf[0] == 'k') {
+		switch (buf[2]) {
+			case '0':
+
+	init_timer(&zipline_kbd_timer);
+	zipline_kbd_timer.function = zipline_kbd_timeout;
+	zipline_kbd_timer.data = 0;
+	
+	/* Initialise scan hardware, request int */
+	clps_writel( (clps_readl(SYSCON1) & ~SYSCON1_KBDSCANMASK)
+		 | KBSC_HI, SYSCON1);
+	zipline_kbd_timer.expires = jiffies + scan_delay;
+	add_timer(&zipline_kbd_timer);
+
+	    break;
+			case '1':del_timer(&zipline_kbd_timer); break;
+		}
+		 keyb_ena = buf[2]-'0'; return count;	
+	}	
+		
+	return -EINVAL;
+}
+
+
+/*
+ * Initialize the keyboard hardware. 
+ * Set all columns high
+ *
+ * Machine dependent parameters:
+ *
+ * fastscan: 		0 = timer based scan for each column
+ *			1 = full scan is done in one timer event
+ * scan_delay:		time between column scans 
+ * 			setup even if you use fastscan (leeds to timer mode)
+ * scan_interval:	time between full scans
+ * 
+ */
+void __init zipline_kbd_init_hw(void)
+{
+
+	/*
+	* put here  machine dependent init stuff 
+	*/
+		fastscan = 1;
+		scan_interval = 50*HZ/1000;
+		scan_delay = 20*HZ/1000;
+		act_scancode = zip_scancode;
+
+	last_column = KBSC_COL0;
+	key_is_pressed = 0;
+
+	/* Register the /proc entry */
+	zipline_keyboard_proc_entry = create_proc_entry("keyboard", 0444,
+		&proc_root);
+	if (zipline_keyboard_proc_entry == NULL)
+	    	printk("Couldn't create the /proc entry for the keyboard\n");
+	else {
+		zipline_keyboard_proc_entry->read_proc = 
+			&zipline_kbd_proc_keyboard_read;
+		zipline_keyboard_proc_entry->write_proc = 
+			&zipline_kbd_proc_keyboard_write;
+	}
+
+	/* Initialize the matrix processing task. */
+	k_translate	= zipline_translate;
+	k_unexpected_up	= zipline_unexpected_up;
+	kbd_process_task.routine = zipline_kbd_process;
+	kbd_process_task.data = 0;
+	
+	/* Setup the timer for keyboard polling */
+	init_timer(&zipline_kbd_timer);
+	zipline_kbd_timer.function = zipline_kbd_timeout;
+	zipline_kbd_timer.data = 0;
+
+	/* Initialise scan hardware */
+	clps_writel( (clps_readl(SYSCON1) & ~SYSCON1_KBDSCANMASK)
+		 | KBSC_HI, SYSCON1);
+	zipline_kbd_timer.expires = jiffies + scan_delay;
+	add_timer(&zipline_kbd_timer);
+
+	printk("zipline keyboard init done\n");
+
+}

