diff -urN -X dontdiff linux-2.4.21/drivers/mtd/maps/physmap.c linux-2.4.21.32/drivers/mtd/maps/physmap.c
--- linux-2.4.21/drivers/mtd/maps/physmap.c	2005-11-04 20:52:16.000000000 -0700
+++ linux-2.4.21.32/drivers/mtd/maps/physmap.c	2005-09-17 21:32:18.000000000 -0600
@@ -12,6 +12,10 @@
 #include <linux/mtd/map.h>
 #include <linux/config.h>
 
+#ifdef CONFIG_MTD_PARTITIONS_MODULE
+#define CONDIF_MTD_PARTITIONS 1
+#endif
+
 #ifdef CONFIG_MTD_PARTITIONS
 #include <linux/mtd/partitions.h>
 #endif
@@ -86,26 +90,39 @@
 #else
 static struct mtd_partition physmap_partitions[] = {
 /* Put your own partition definitions here */
-#if 0
+#ifdef ZIPIT_DANGER_PHYSMAP
+#define RESCUE_MASK 0
+#define NONRESCUE_MASK 0
+#else
+#define RESCUE_MASK_WRIEABLE
+#define NONRESCUE_MASK 0
+#endif
+#if 1
 	{
-		name:		"bootROM",
-		size:		0x80000,
+		name:		"boot loader",
+		size:		0x2000, /* 8k */
 		offset:		0,
-		mask_flags:	MTD_WRITEABLE,  /* force read-only */
+		mask_flags:	RESCUE_MASK,  /* force read-only */
 	}, {
 		name:		"zImage",
-		size:		0x100000,
-		offset:		MTDPART_OFS_APPEND,
-		mask_flags:	MTD_WRITEABLE,  /* force read-only */
+		size:		0x8e000, /* 568k */
+		offset:		0x2000,
+		mask_flags:	RESCUE_MASK,  /* force read-only */
 	}, {
-		name:		"ramdisk.gz",
-		size:		0x300000,
-		offset:		MTDPART_OFS_APPEND,
-		mask_flags:	MTD_WRITEABLE,  /* force read-only */
+		name:		"rescue/initrd"
+		size:		0xc0000, /* 768k */
+		offset:		0x90000,
+		mask_flags:	RESCUE_MASK, /* force read-only */
+	}, { 
+		name:		"user area A",
+		size:		0xa0000, /* 640k */
+		offset:		0x150000,
+		mask_flags:	NONRESCUE_MASK,  /* read-write */
 	}, {
-		name:		"User FS",
-		size:		MTDPART_SIZ_FULL,
-		offset:		MTDPART_OFS_APPEND,
+		name:		"user area B",
+		size:		0x10000, /* 64k */
+		offset:		0x1f0000,
+		mask_flags:	NONRESCUE_MASK, /* read-write */
 	}
 #endif
 };
@@ -122,7 +139,7 @@
 
        	printk(KERN_NOTICE "physmap flash device: %x at %x\n", WINDOW_SIZE, WINDOW_ADDR);
 #ifdef CONFIG_ARCH_ZIPLINE 
-	physmap_map.map_priv_1 = (unsigned long)ioremapi_nocache(WINDOW_ADDR, WINDOW_SIZE);
+	physmap_map.map_priv_1 = (unsigned long)ioremap_nocache(WINDOW_ADDR, WINDOW_SIZE);
 #else
 	physmap_map.map_priv_1 = (unsigned long)ioremap(WINDOW_ADDR, WINDOW_SIZE);
 #endif
@@ -156,9 +173,23 @@
 			printk(KERN_NOTICE 
 			       "Using physmap partition definition\n");
 			add_mtd_partitions (mymtd, physmap_partitions, NUM_PARTITIONS);
+		}  else  {
+		        printk(KERN_NOTICE
+				"No partitions defined in physmap.c\n");
 		}
 
 #endif
+		/* Now reset whole-chip device to read-only status */
+		
+#ifndef ZIPIT_DANGER_PHYSMAP
+		printk(KERN_NOTICE
+			       "Safe Zipit MTD mode: Marking whole-block device and kernel/initrd R/O\n");
+		mymtd->flags = MTD_CAP_ROM;
+#else
+		printk(KERN_NOTICE
+			       "DANGER Zipit MTD mode: all flash partitions writeable\n");
+#endif
+
 #endif
 		return 0;
 	}

