diff -urN -X dontdiff linux-2.4.21/drivers/sound/wm8751l.c linux-2.4.21.32/drivers/sound/wm8751l.c
--- linux-2.4.21/drivers/sound/wm8751l.c	1969-12-31 17:00:00.000000000 -0700
+++ linux-2.4.21.32/drivers/sound/wm8751l.c	2005-04-15 19:01:43.000000000 -0600
@@ -0,0 +1,235 @@
+/*
+ * Wolfson Micro WM8751L Stereo DAC I2C Driver
+ *   For the Zipit IM device
+ *
+ * (C) 2005 Ken McGuire
+ * 
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 2 as
+ * published by the Free Software Foundation.
+ *
+ * Based on original work by:
+ * 
+ * CS43L42 DAC on the EDB7312 Cogent eval board
+ *
+ * (C) 2003 Heeltoe Consulting, http://www.heeltoe.com
+ * (c) 2003 Nucleus Systems, Petko Manolov <petkan@nucleusys.com>
+ *
+ * linux/drivers/sound/wm8751l.c
+ */
+
+
+#include <linux/module.h>
+#include <linux/config.h>
+#include <linux/kernel.h>
+#include <linux/types.h>
+#include <linux/init.h>
+#include <asm/io.h>
+#include <asm/arch/hardware.h>
+#include <asm/hardware/ep7312.h>
+#include <asm/hardware/clps7111.h>
+#include <asm/delay.h>
+
+ /* Shift the address over to make room for the r/w bit */
+#define ADDR_WM8751L	(0x1b << 1)
+/* direction and i/o bits have the same offset */
+#define	SCL		(1 << 7)
+#define SDA		(1 << 6)
+
+static void make_scl_sda_outputs(void)
+{
+	unsigned char tmp;
+	
+	tmp = clps_readb(PBDDR);
+	tmp |= (SCL | SDA);
+	clps_writeb(tmp, PBDDR);
+}
+
+static void make_sda_input(void)
+{
+	unsigned char tmp;
+
+	tmp = clps_readb(PBDDR);
+	tmp &= ~SDA;
+	clps_writeb(tmp, PBDDR);
+}
+
+static void set_scl(int val)
+{
+	unsigned char tmp;
+	
+	tmp = clps_readb(PBDR);
+	if (val)
+		tmp |= SCL;
+	else
+		tmp &= ~SCL;
+	clps_writeb(tmp, PBDR);
+}
+
+static void set_sda(int val)
+{
+	unsigned char tmp;
+	
+	tmp = clps_readb(PBDR);
+	if (val)
+		tmp |= SDA;
+	else
+		tmp &= ~SDA;
+	clps_writeb(tmp, PBDR);
+}
+
+static int get_sda(void)
+{
+	unsigned char tmp;
+	
+	tmp = clps_readb(PBDR);
+	return (tmp & SDA) ? 1 : 0;
+}
+
+
+static void clock_byte_out(int b, int set_output)
+{
+	int i, ack;
+
+	for (i = 0; i < 8; i++) {
+		udelay(3);
+		set_sda(b & 0x80);
+		udelay(3);
+		set_scl(1);
+		udelay(5);	/* Tlow == 4.7 uS */
+		set_scl(0);
+		b <<= 1;
+	}
+	/* ack bit */
+	make_sda_input();
+	udelay(5);
+	set_scl(1);
+	udelay(2);
+	ack = get_sda();
+	udelay(3);
+	set_scl(0);
+//	printk("ack= %d\n", ack);
+
+
+	if (set_output) {
+		make_scl_sda_outputs();
+	}
+}
+
+
+static int wm8751l_i2c_write(int addr, int reg, int value)
+{
+
+//    printk("dac write reg= %d, val= %d\n", reg, value);
+
+	set_sda(1);
+	set_scl(1);
+	udelay(5);
+	set_sda(0);
+	udelay(4);
+	set_scl(0);
+	clock_byte_out(addr, 1);
+	clock_byte_out(reg, 1);
+	clock_byte_out(value, 1);
+	set_sda(1);
+	set_scl(1);
+	udelay(5);
+
+	return 0;
+}
+
+int wm8751l_write(int reg, int value)
+{
+    wm8751l_i2c_write(ADDR_WM8751L, reg, value);
+
+	return 0;
+}	
+
+int setup_wm8751l(void)
+{
+	volatile long u;
+	volatile char port_b;
+
+	printk("wm8751l: init dac\n");
+	port_b = clps_readb(PBDR) & ~(SCL | SDA);
+	
+
+	/* make SDA & SCL outputs */
+	make_scl_sda_outputs();
+
+
+	/* delay while the DAC initializes */
+	for(u = 0; u < 15; u++)
+		udelay(5);
+
+
+    /*  wm8751 init to Zipit compatible values  */
+
+	/* reset the DAC */
+
+    	wm8751l_i2c_write(ADDR_WM8751L, 0x1e, 0x00);
+
+	/* power on the DAC R25 PWR Management  */    
+
+	wm8751l_i2c_write(ADDR_WM8751L, 0x33, 0xc0);
+
+	/* delay while the DAC initializes */
+	for(u = 0; u < 15; u++)
+		udelay(5);
+
+	/* R2 LOUT1 */
+
+	wm8751l_i2c_write(ADDR_WM8751L, 0x05, 0x60);
+
+	/* R3 ROUT1 */
+
+	wm8751l_i2c_write(ADDR_WM8751L, 0x07, 0x60);
+
+	/* R5 DAC Control */
+
+	wm8751l_i2c_write(ADDR_WM8751L, 0x0a, 0x00);
+
+	/* delay while the DAC initializes */
+	for(u = 0; u < 15; u++)
+		udelay(5);
+
+	/* R7 Audio Interface */
+
+	wm8751l_i2c_write(ADDR_WM8751L, 0x0e, 0x01);
+
+	/* R8 Clocking */
+
+	wm8751l_i2c_write(ADDR_WM8751L, 0x10, 0x20);
+
+	/* R26 Pwr Management 2 */
+
+	wm8751l_i2c_write(ADDR_WM8751L, 0x35, 0xf8);
+
+	/*  R24 Additional 2  */
+
+	wm8751l_i2c_write(ADDR_WM8751L, 0x30, 0x68);
+
+	/* R34 Left Mix 1 */
+
+	wm8751l_i2c_write(ADDR_WM8751L, 0x45, 0x00);
+
+	/* R37 Right Mix 1 */
+
+	wm8751l_i2c_write(ADDR_WM8751L, 0x4b, 0x00);
+
+	/* R10 Left Digital Volume */
+
+	wm8751l_i2c_write(ADDR_WM8751L, 0x15, 0xff);
+
+	/* R11 Right Digital Volume */
+
+	wm8751l_i2c_write(ADDR_WM8751L, 0x17, 0xff);
+
+	/* restore original port B value */
+	port_b |= clps_readb(PBDR);
+	clps_writeb(port_b, PBDR);
+	printk("wm8751l: init dac complete\n");
+
+	return 0;
+}

